use strict;
use warnings;
use Test::More;

# generated by Dist::Zilla::Plugin::Test::PodSpelling 2.006008
use Test::Spelling 0.12;
use Pod::Wordlist;


add_stopwords(<DATA>);
all_pod_files_spelling_ok( qw( bin lib  ) );
__DATA__
unindexed
Bam
Baz
CTRL
Distroprefs
SIGKILL
cpanreporter
defaultdeps
distroprefs
kurila
minicpanrc
prerelease
titlebar
untarring
urllist
David
Golden
dagolden
Alexandr
Ciornii
alexchorny
Christian
Walde
walde
gregor
herrmann
gregoa
lib
CPAN
Reporter
Smoker
